// Demo-Klasse fr Template-Anwendungen
// Erfllt die Regel der Grossen 5

#include <string>
#include <iostream>

// Makro fr Debug-Ausgaben
//#define TRACE_METH
#ifdef TRACE_METH
    #define TRACE(T_)	std::cout <<  "Obj-Nr" << num << ": " << T_ << ',' << text << std::endl
#else
    #define TRACE(T_)
#endif


class Demo
{
    std::string text;
    static int count;
    int num;
public:
    Demo() noexcept;
    Demo(const char* pText) noexcept;
    Demo(const Demo& obj) noexcept;
	Demo(Demo&& obj) noexcept;
    virtual ~Demo();
    Demo& operator = (const Demo& rhs);
	Demo& operator = (Demo&& rhs);
    bool operator < (const Demo& rhs) const;
    bool operator == (const Demo& rhs) const;
    friend std::ostream& operator << (std::ostream& os, const Demo& obj);
    const std::string& GetText() const
    {
        return text;
    }
};

// ==========================================================================
// In der Praxis sollten Sie in einer Header-Datei keine Memberfkt. definieren
// da es sonst beim Einbinden der Header-Datei in mehreren C++ Quelldateien
// zu doppelt-definierten Memberfunktionen kommt. Dies dient hier einzig und allein
// zur Vereinfachung des Compile-Vorgangs und entspricht daher in keiner
// Weise der alltglichen Praxis!
// ==========================================================================

// statische Objektzhler
decltype(Demo::count) Demo::count=0;

// ctors
Demo::Demo() noexcept
{
    num = ++count;
    TRACE("std-ctor");
}
Demo::Demo(const char* pText) noexcept: text(pText)
{
    num = ++count;
    TRACE("ctor");
}
Demo::Demo(Demo&& obj) noexcept
{
    num = ++count;
	text = std::move(obj.text);
	TRACE("move ctor");
}
// Hinweis: ohne Ausgabe koennte der copy-ctor auch
// als default copy-ctor definiert werden
Demo::Demo(const Demo& obj)noexcept: text(obj.text)
{
    if (&obj == this)
        TRACE ("copy-ctor self!!");
    num = ++count;
    TRACE("copy-ctor");
}
// dtor
Demo::~Demo()
{
    TRACE("dtor");
}
// Zuweisungsoperator
Demo& Demo::operator = (const Demo& rhs)
{
    if (&rhs==this)
        return *this;
    text = rhs.text;
    TRACE("operator=");
    return *this;
}
Demo& Demo::operator = (Demo&& rhs)
{
	if (this == &rhs)
		return *this;
	text = std::move(rhs.text);
	num = rhs.num;
	TRACE("move operator=");
	return *this;
}

bool Demo::operator < (const Demo& rhs) const
{
    TRACE("operator<");
    return text < rhs.text;
}
bool Demo::operator == (const Demo& rhs) const
{
    TRACE("operator==");
    return text == rhs.text;
}
// Ausgabeoperator
std::ostream& operator << (std::ostream& os, const Demo& obj)
{
    std::cout << obj.text;
    return os;
}
