#ifndef OBJECT_H__
#define OBJECT_H__

#include <iostream>
#include <string>

// Makro fr Debug-Ausgabe
#ifdef TRACE_METH
    #define TRACE(_T)   std::cout << _T << " Obj-Nr:" << num << std::endl
#else
    #define TRACE(_T)
#endif

// Object Klasse
class Object
{
   std::string text;  // Objektdatum
   static int count;  // Objektzhler
   int num;           // aktuelle Objektnummer
public:
   Object();                   // Standard-ctor
   Object(const char* pText);  // ctor
   Object(const Object& obj);  // copy-ctor
   Object(Object&& obj);       // move-ctor
   virtual ~Object();          // dtor
   // Zuweisungsoperator
   Object& operator = (const Object& rhs);
   // Move-Operator =
   Object& operator = (Object&& rhs);
   // Vergleichsoperatoren
   bool operator < (const Object& rhs) const;
   bool operator == (const Object& rhs) const;
   // Restl. Vergleichsops auf Basis von < und == definieren
   bool operator != (const Object& rhs) const
   {
        return !(*this == rhs);
    }
    bool operator > (const Object& rhs) const
    {
        return rhs < *this;
    }
    bool operator <= (const Object& rhs) const
    {
        return !(rhs < *this);
    }
   bool operator >= (const Object& rhs) const
   {
        return !(*this < rhs);
    }
   // Ausgabeoperator
   friend std::ostream& operator << (std::ostream& os, const Object& obj);
};
// Statisches Datum definieren
decltype(Object::count) Object::count=0;
// ctor-Definitionen
inline Object::Object()
{
   num = ++count;
   TRACE("std-ctor");
}
inline Object::Object(const char* pText)
{
   num = ++count;
   text = pText;
   TRACE("ctor");
}
inline Object::Object(const Object& obj)
{
   num = ++count;
   text = obj.text;
   TRACE("copy-ctor");
}
inline Object::Object(Object&& obj)
{
   text = std::move(obj.text);
   num = obj.num;
   obj.text.clear();
   obj.num = 0;
   TRACE("move-ctor");
}
// dtor Definition
inline Object::~Object()
{
   TRACE("dtor");
}
// berladene Operatoren definieren
inline Object& Object::operator = (const Object& rhs)
{
   if (&rhs==this)
      return *this;
   text = rhs.text;
   TRACE("operator=");
   return *this;
}
inline Object& Object::operator = (Object&& rhs)
{
   text = std::move(rhs.text);
   num = rhs.num;
   rhs.text.clear();
   rhs.num = 0;
   TRACE("move operator=");
   return *this;
}
inline bool Object::operator < (const Object& rhs) const
{
   return text < rhs.text;
}
inline bool Object::operator == (const Object& rhs) const
{
   return text == rhs.text;
}
inline std::ostream& operator << (std::ostream& os, const Object& obj)
{
   std::cout << obj.text;
   return os;
}
#endif
