#include <iostream>
#include <string>

using std::cout;
using std::endl;
using std::string;

// Definition der allg. Fensterklasse
class Window
{
    short xPos, yPos; // Fensterposition
    short width, height; // Fenstergrösse
    string title; // Fenstertitel
public:
    // ctor
    Window(short x, short y, short w, short h,
           const char* const pT);
    // Zuweisung von Fenster-Objekten unterbinden
    Window& operator = (const Window&) = delete;
    // Memberfunktionen zum Verschieben und Zeichnen des Fensters
    void MoveWindow(short x, short y);
    void Draw() const;
};
// Defintion der Memberfunktionen
// Konstruktor
Window::Window(short x, short y, short w, short h,
               const char* const pT):
               xPos(x), yPos(y), width(w),
               height(h), title(pT)
{}
// Verschiebt Fenster
inline void Window::MoveWindow(short x, short y)
{
    xPos = x; yPos = y;
}
// Zeichnet Fenster
void Window::Draw() const
{
    cout << "Pos (" << xPos << ',' << yPos << "), Groesse (" <<
            width << ',' << height << ")\n";
    cout << "Titel: " << title << endl;
}

// Abgeleitete Klasse für Rahmenfenster
class FrameWin: public Window
{
public:
    // enums für Rahmenfarbe, müssen public sein da
    // sie im ctor benötigt werden
    enum eFColor{ROT=0xFF0000,GRUEN=0x00FF00,BLAU=0x0000FF};
private:
    eFColor frameColor; // Rahmenfarbe
public:
    // ctor
    FrameWin(short x, short y,
             const char* const pT, eFColor eFC);
    // Memberfunktion zum Zeichnen des Fensters
    void Draw() const;
};
// Definition der Memberfunktionen
// Konstruktor
FrameWin::FrameWin(short x, short y,
                   const char* const pT, eFColor eFC)
    :Window(x, y, 640, 480, pT), frameColor(eFC)
{}
// Zeichnet Fenster
void FrameWin::Draw() const
{
    // 'Normales' Fenster zeichnen
    Window::Draw();
    // Rahmenfarbe darstellen
    cout << "Rahmenfarbe: ";
    switch (frameColor)
    {
    case ROT:
        cout << "rot";
        break;
    case BLAU:
        cout << "blau";
        break;
    case GRUEN:
        cout << "grün";
    }
    cout << endl << endl;
}

// main() Funktion
int main()
{
    // Ein normales Fenster und ein Rahmenfenster erstellen
    Window myWin(10,10,800,600,"Normales Fenster");
    // Beachten Sie die Angabe des enum-Parameter (letzter Parameter)!
    FrameWin myFrameWin(40,40,"Rahmenfenster",FrameWin::BLAU);

    // Beide Fenster darstellen
    myWin.Draw();
    myFrameWin.Draw();
    // Rahmenfenster verschieben
    // MoveWindow(...) ist Memberfunktion von Window!
    myFrameWin.MoveWindow(0,0);
    // Rahmenfenster erneut darstellen
    myFrameWin.Draw();
}
