// Dateien einbinden
#include <iostream>

using std::cout;
using std::endl;

// Funktionstemplate zum Tauschen von Werten beliebigen Datentyps
// Das Funktionstemplate verwendet die move() Funktion, d.h.
// sollten Objekte getauscht werden, so sollten diese die entsprechenden
// Move-Memberfunktionen definieren
template <typename T>
void Swap(T& val1, T& val2)
{
    T temp(std::move(val1)); // Move-Konstruktor verwenden!
    val1 = std::move(val2); // Damit könnten Sie sogar Objekt tauschen
    val2 = std::move(temp); // wenn diese den Operator = definieren
}

// Funktionstemplate zum Sortieren von
// beliebigen Datentypen innerhalb eines Feldes
// pValues ist der Zeiger auf den Beginn des Datenfeldes
// und nNoOfValues enthält die Anzahl der Daten
template <typename T>
void Sort(T pValues, int const noOfValues)
{
    bool changed; // Tauschflag

    // Tauschschleife
    do
    {
        // Tauschflag löschen
        changed = false;
        // Alle Elemente vergleichen
        for (auto index=0; index<noOfValues-1; index++)
        {
            // Falls getauscht werden muss
            if (pValues[index]>pValues[index+1])
            {
                // Werte tauschen
                Swap(pValues[index],pValues[index+1]);
                // Tauschflag setzen
                changed = true;
            }
        }
    } while (changed);
    // Schleife so lange durchlaufen, bis nicht mehr getauscht wurde
}

// main() Funktion
int main()
{
    // Felder mit den zu sortierenden Daten
    long longArray[] {1,-10,-2,20};
    double doubleArray[] {1.1, 0.9, -1.2, 5.5};

    // Anzahl der Elemente in den Feldern berechnen!
    constexpr auto noOfLongs = sizeof(longArray)/sizeof(longArray[0]);
    constexpr auto noOfDouble = sizeof(doubleArray)/sizeof(doubleArray[0]);

    // unsortiertes long-Feld ausgeben
    cout << "Unsortierte long:\n";
    for (auto elem: longArray)
       cout << elem << ", ";
    cout << endl;
    // -> erzeugt Funktion: void Sort(long*, int);
    Sort(longArray,noOfLongs);
    // sortiertes long-Feld ausgeben
    cout << "Sortierte long:\n";
    for (auto elem: longArray)
       cout << elem << ", ";
    cout << endl << endl;

    // unsortiertes double Feld ausgeben
    cout << "Unsortiert double:\n";
    for (auto elem: doubleArray)
       cout << elem << ", ";
     cout << endl;
    // -> erzeugt Funktion: void Sort(double*, int);
    Sort(doubleArray,noOfDouble);
    // sortiertes double-Feld ausgeben
    cout << "Sortierte double:\n";
    for (auto elem: doubleArray)
       cout << elem << ", ";
    cout << endl << endl;

}
