#include <iostream>
#include <typeinfo>

using std::cout;
using std::endl;

// Funktionstemplate zur byteweisen 'Übertragung' eines
// Wertes in ASCII-Form
template <typename T>
void SendTo(std::ostream& os, T value)
{
    // Datentyp ausgeben
    os << "sending bytes: ";
    os << typeid(T).name() << ' ';
    // char-Zeiger auf Datenbeginn aufsetzen
    const char* p = reinterpret_cast<const char*>(&value);
    // Alle Bytes durchlaufen
    for (size_t i=0; i<sizeof(T); i++)
    {
        // Low-Nibble nach ASCII konvertieren
        char lowNibble = (*p)&0x0f;
        lowNibble = (lowNibble>9) ? lowNibble+0x57 : lowNibble|0x30;
        // High-Nibble nach ASCII konvertieren
        char highNibble = ((*p)>>4)&0x0f;
        highNibble = (highNibble>9) ? highNibble+0x57 : highNibble|0x30;
        // Datum in ASCII-Form 'übertragen'
        os << highNibble << lowNibble << ' ';
        // Zeiger auf nächstes Byte
        p++;
    }
    cout << endl;
}

// main() Funktion
int main()
{
    // Auszugebende Daten
    int intVal = 10;
    char charVal = 'A'; // entspricht 'A'

    // Aufruf der Template-Funktion
    // Der Template-Datentyp ergibt sich aus dem Datentyp des Parameters
    SendTo(cout,intVal);
    SendTo(cout,charVal);

    // Aufruf der Template-Funktion mit Literalen
    // Standardmässig werden Literale als ints behandelt
    // Soll SendTo() mit anderem Datentyp aufgerufen werden,
    // so kann entweder der Datentyp explizit festgelegt werden
    // (2. Aufruf) oder aber der Template-Datentyp explizit
    // angegeben werden (3. + 4. Aufruf)
    SendTo(cout,0x61);
    SendTo(cout,0x61L);
    SendTo<char>(cout,0x61);
    SendTo<short>(cout,0x1234);
}
