#include <iostream>
#include <string>
#include <tuple>
#include <iomanip>
#include <new>

using std::cout;
using std::endl;
using std::string;
using namespace std::string_literals;

template <typename T>
class MyArray
{
    T* tArray;
    int actSize;
public:
    // ctor
    MyArray (int size)
    {
        tArray = new T[size];       // Feld anlegen
        actSize = size;             // und Groesse merken
    }
    // dtor
    ~MyArray()
    {
        delete [] tArray;           // Feld loeschen
    }
    // Kopieren und zuweisen verhindern
    MyArray(MyArray&) = delete;
    MyArray(MyArray&&) = delete;
    MyArray& operator = (const MyArray&) = delete;
    MyArray& operator = (const MyArray&&) = delete;
    // Daten setzen
    void SetData (int index, const T& data)
    {
        if ((index<0) || (index>=actSize))
            throw std::out_of_range("Index ausserhalb erlaubten Bereich");
        tArray[index] = data;
    }
    // Daten auslesen
    T& GetData (int index) const
    {
        if ((index<0) || (index>=actSize))
            throw std::out_of_range("Index ausserhalb erlaubten Bereich");
        return tArray[index];
    }
};
// Alias fuer tuple
// departure dient zum Abspeichern der Daten eines Fluges
// Stunde, Minute, Flugnummer und Zielort
using departure = std::tuple<short,short,string,string>;
// Feldgroesse
constexpr auto ttSize =4;

// main() Funktion
int main()
{
    // Tuple-Feld definieren
    MyArray<departure> timeTable(ttSize);
    // und Daten uebernehmen
    timeTable.SetData(0,{11, 30, "X3 2345"s, "Palma de Mallorca"s});
    timeTable.SetData(1,{11, 45, "LH 3453"s, "New York"s});
    timeTable.SetData(2,{11, 50, "DE 264"s, "Teneriffe South"s});
    timeTable.SetData(3,{12, 05, "4U 9876"s, "Berlin"s});

    // Daten wieder auslesen
    cout << std::setfill('0');
    for (auto index=0; index<ttSize; index++)
    {
        auto [hour,minute,fnumber,destination] = timeTable.GetData(index);
        cout << std::setw(2) << hour << ':' << std::setw(2) << minute << "\tFlug: "
            << fnumber << "\tZiel: " << destination << endl;
    }
}
