#include <iostream>
#include <string>

using std::cout;
using std::endl;
using std::string;

// Definition der Template-Klasse Stack
// Defaultmässig wird ein Stack für 5 Einträge erstellt
template <typename T, int SIZE=5>
class Stack
{
  private:
     short stackPtr = 0; // Stackindex (Stackpointer)
     T data[SIZE]; // Zeiger auf Stackobjekte
  public:
     void Push(const T&);
     void Pop(T&);
};
// Definition der Memberfunktionen
// Legt Datum auf dem Stack ab
template <typename T, int SIZE> void Stack<T,SIZE>::Push(const T& nV)
{
    // Falls Stack schon voll ist
    if (stackPtr==SIZE)
        throw "Stack ist voll";
    // Datum ablegen
    data[stackPtr++] = nV;
}
// Liest Datum vom Stack aus
template <typename T, int SIZE> void Stack<T,SIZE>::Pop(T& nV)
{
    // Falls Stack leer
    if (stackPtr==0)
        throw "Stack ist leer";
    // Datum holen
    nV = data[--stackPtr];
}

// Definition der Demo-Klasse Window
// Window dient nur zum Beweis, dass auf dem Stack jetzt auch
// Objekte bzw. Zeiger auf Objekte abgelegt werden können
class Window
{
    string title;
  public:
    Window(const char *const pT): title(pT)
    {}
    Window() = default;
    Window& operator = (const Window& obj2);
    friend std::ostream& operator << (std::ostream&, const Window&);
};
// Definition der Memberfunktionen
// Überladener Zuweisungsoperator
Window& Window::operator =(const Window& obj2)
{
    title = obj2.title;
    return *this;
}
// Überladener Ausgabeoperator << für die Window-Klasse
std::ostream& operator << (std::ostream& os, const Window& obj)
{
     os << obj.title;
     return os;
}

// main() Funktion
int main()
{
    // Short-Stack definieren für 5 short-Wert (Defaultgrösse)
    Stack<short> shortStack;

    try
    {
        // Short-Stack komplett füllen
        for (auto index=1;;index++)
        {
            // Wert auf dem Stack abgelegen
            shortStack.Push(index);
            cout << index << " abgelegt.\n";
        }
    }
    catch(const char* err)
    {
        cout << err << endl;
    }

    // Short-Stack wieder auslesen
    try
    {
        for (;;)
        {
            short val;
            // Falls Wert ausgelesen wurde
            shortStack.Pop(val);
            cout << val << " wieder geholt.\n";
        }
    }
    catch(const char* err)
    {
        cout << err << endl;
    }

    // Stack für 10 Window-Objekte definieren
    Stack<Window,10> windowStack;
    // Window-Objekte auf Stack ablegen
    // ACHTUNG! Hier wird ein temp. Window-Objekt erstellt
    // welches dann an Push() übergeben wird. Push() ruft
    // den Zuweisungsoperator von Window auf um das temp.
    // Objekt in den Stack zu übernehmen. Nach der Rückkehr
    // von Push() wird das temp. Objekt wieder gelöscht!
    cout << "Fuelle Window-Stack\n";
    windowStack.Push(Window("Fenster 1"));
    windowStack.Push(Window("Fenster 2"));
    windowStack.Push(Window("Fenster 3"));

    // Nun Window-Objekte wieder vom Stack holen
    cout << "Hole Window-Daten vom Stack\n";
    // Window-Objekt erstellen
    // Wird von Pop() mit dem akt. Stack-Objekt
    // 'ausgefüllt' (Aufruf des Zuweisungsoperators)
    try
    {
        for (;;)
        {
            Window actWindow;
            windowStack.Pop(actWindow);
            cout << actWindow << endl;
        }
    }
    // Ausnahmebehandlung, nun einmal leer
    catch(...)
    {
    }
}
