#include <iostream>
#include <stdexcept>
#include "demo.h"

using std::cout;
using std::endl;

// Definition der Template-Klasse SArray
// Defaultmässig wird Safe-Array für int-Daten erstellt
template <typename T=int>
class SArray
{
    short size; // Grösse des Feldes
    T *pData; // Zeiger auf Datenfeld
public:
    SArray(short size);
    ~SArray();
    T& operator[] (int index);
};
// Definition der Memberfunktionen
// Konstruktor
template <typename T>
SArray<T>::SArray(short size_)
{
    // Grösse des Feldes merken
    size = size_;
    // Feld anlegen
    pData = new T[size];
}
// Destruktor
template <typename T>
SArray<T>::~SArray()
{
    // Feld freigeben
    delete [] pData;
}
// Überladener Indexoperator
template <typename T>
T& SArray<T>::operator [](int index)
{
    // Falls Index kleiner 0 ist
    if (index<0)
        throw std::range_error("Index kleiner 0");
    // Falls Index über das Feld hinausgreift
    if (index>=size)
        throw std::range_error("Index ueberschreitet obere Grenze");
    // Datum zurückgeben
    return pData[index];
}

// main() Funktion
int main()
{

    constexpr auto ARRAYSIZE = 10; // Feldgrösse

    // Safe-Array mit Standard-Datentyp int anlegen
    SArray<> shortArray(ARRAYSIZE);
    // Safe-Array füllen
    try
    {
        for (auto index=0; index<ARRAYSIZE; index++)
            shortArray[index] = index;
        // Safe-Array wieder auslesen
        for (auto index=0; index<ARRAYSIZE; index++)
            cout << index << ". Wert: " << shortArray[index] << endl;
        // Versuch das Element mit dem Index -1 zu beschreiben
        cout << "Versuch das Element -1 zu beschreiben!\n";
        shortArray[-1] = 111;
    }
    catch (std::exception& ex)
    {
        cout << ex.what() << endl;
    }

    // Safe-Array für Demo-Objekte erstellen
    SArray<Demo> demoArray(3);
    try
    {
        // Erstes und letztes Feldelement belegen
        demoArray[0] = Demo("Demo-Objekt 1");
        demoArray[2] = Demo("Demo-Objekt 2");
        // Demo-Objekte im Safe-Array ausgeben
        cout << demoArray[0] << endl;
        cout << demoArray[2] << endl;
        // Nun über obere Grenze hinausgreifen
        cout << demoArray[5] << endl;
    }
    catch (std::exception& ex)
    {
        cout << ex.what() << endl;
    }
}
