#include <iostream>
#include <string>
#include <iomanip>
#include <unordered_map>
#include <vector>
#include <algorithm>

using std::cout;
using std::endl;
using namespace std::string_literals;

// Aktiendepot
// std::string enthaelt den Aktienname
// std::vector enthaelt die Aktienkurse
using adata = std::unordered_map<std::string, std::vector<float>>;
adata aktien;

// Gibt den Aktienname und den Kurs aus
// Wird ueber for_each() aufgerufen
// adata::value_type ist ein std::pair und enthaelt ein
// Element aus der unordered_map des Aktiendepots
void PrintAktien (const adata::value_type& aktie)
{
    cout << aktie.first << ": " << aktie.second.back() << endl;
}

int main()
{
    // Ausgabe mit 2 Nachkommastellen
    cout << std::fixed << std::setprecision(2);

    // Aktien anlegen
    aktien["Addidas AG"s].push_back(275.00f);
    aktien["Allianz SE"s].push_back(212.00f);
    aktien["BASF SE"s].push_back(64.00f);
    aktien["Bayer AG"s].push_back(68.00f);
    // und ausgeben
	cout << "1. Tag:\n";
    std::for_each(aktien.begin(), aktien.end(),PrintAktien);
    // Summe der Aktien berechnen
    // Letzter Eintrag im Kurs-Vektor enthaelt aktuellen Kurs
    float sum = 0.00f;
    for_each(aktien.begin(), aktien.end(),
                [&sum](const adata::value_type& aktie)
                 { sum += aktie.second.back(); }
             );
    cout << "Depotwert: " << sum << endl;

    // Neuen Aktienkurs eintragen
    aktien["Addidas AG"s].push_back(274.00f);
    aktien["Allianz SE"s].push_back(214.25f);
    aktien["BASF SE"s].push_back(65.00f);
    aktien["Bayer AG"s].push_back(65.30f);
    // und ausgeben
    cout << "\n2. Tag:\n";
    std::for_each(aktien.begin(), aktien.end(),PrintAktien);
    // Summe der Aktien berechnen
    sum = 0.0f;
    for_each(aktien.begin(), aktien.end(),
                [&sum](const adata::value_type& aktie)
                 { sum += aktie.second.back(); }
             );
    cout << "Depotwert: " << sum << endl;

    // Neuen Aktienkurs eintragen
    aktien["Addidas AG"s].push_back(272.30f);
    aktien["Allianz SE"s].push_back(215.75f);
    aktien["BASF SE"s].push_back(65.10f);
    aktien["Bayer AG"s].push_back(68.00f);
    // und ausgeben
    cout << "\n3. Tag:\n";
    std::for_each(aktien.begin(), aktien.end(),PrintAktien);
    // Neue Summe der Aktien berechnen
    sum = 0.00f;
    for_each(aktien.begin(), aktien.end(),
                [&sum](const adata::value_type& aktie)
                 { sum += aktie.second.back(); }
             );
    cout << "Depotwert: " << sum << endl;

    // Aktienkurs-Verlauf ausgeben
    cout << "\nKursverlaeufe:\n";
    for_each(aktien.begin(), aktien.end(),
                [](const adata::value_type& aktie)
                 {
                     cout << aktie.first << ": ";
                     for (const auto& val: aktie.second)
                        cout << val << ',';
                     cout << endl;
                 }
             );
}
