#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <string>
#include <vector>
#include <tuple>
#include <algorithm>

using std::cout;
using std::endl;

// Dateiname
constexpr const char* FILENAME = "verbrauch.txt";
// Tuple zum Abspeiohern der Tankdaten
// 1. Wert: km-Stand
// 2. Wert: getankte Liter
// 3. Wert: Verbrauch 1tr/100km
using data = std::tuple<unsigned int, float, float>;

// Tankdaten
std::vector<data> verbrauch;

int main()
{
    // Datei mit Tankdaten oeffnen
    std::ifstream infile(FILENAME);
    if (!infile)
    {
        cout << "Datei " << FILENAME << " nicht gefunden!\n";

    }
    // Tankdaten einlesen
    do
    {
        unsigned int km;    // akt. km-Stand
        float tank;         // getankte ltr.
        infile >> km >> tank;
        if (infile.eof())
            continue;
        // Tankdaten im Vektor ablegen, Verbrauch mit 0.0 belegen
        verbrauch.emplace_back(km,tank,0.0f);
    } while (!infile.eof());
    // Datei schliesen
    infile.close();

    // vorheriger km-Stand
    unsigned int km_alt = 0;
    // Tankdaten durchlaufen um Verbrauch zu berechnen
    // Verbrauch wird in einer Lambda-Funktion berechnet
    std::for_each(verbrauch.begin(), verbrauch.end(),
              // Lambda-Funktion, erhaelt vorherigen km-Stand
              // per Referenz und um Parameter akt. Tankdaten
              // Der 1. Eintrag enthaelt nur km-Stand bei Uebergabe
              // Fuer nachfolgende Eintraege Verbrauch berechnen
              // und im Vektor ablegen. Anschließend den akt.
              // km-Stand merken
             [&km_alt] (data& eintrag)
                  {
                      if (km_alt == 0)
                      {
                          km_alt = std::get<0>(eintrag);
                      }
                      else
                      {
                          std::get<2>(eintrag) = std::get<1>(eintrag) /
                              (std::get<0>(eintrag)-km_alt)*100.0f;
                          km_alt = std::get<0>(eintrag);
                      }
                  }
             );

    // Ausgabe der Tankdaten inkl. Verbrauch (auf 2 Dezimalstellen)
    cout << std::fixed << std::setprecision(2) << "Tankfuellung:\n";
    for (auto [km, tank, verbr]: verbrauch)
        cout << "km: " << km << ", getankt: "
            << tank << ", Verbrauch: " << verbr << endl;

    // Minimalen und maximalen Verbrauch ermitteln mittels
    // Lambda-Funktion, da der Verbrauch an 3. Stelle im
    // Vektor steht (Standard-Vergleich erfolgt mit 1. Stelle)
    auto minverbr = std::minmax_element(verbrauch.begin()+1, verbrauch.end(),
                                        [] (const data& lhs, const data& rhs) -> bool
                                        {
                                            return std::get<2>(lhs) < std::get<2>(rhs);
                                        });
    // Ermittelten Min/Max Verbrauch ausgeben
    cout << "\nMin. Verbrauch: " << std::get<2>(*minverbr.first)
         << ", Max. Verbrauch: " << std::get<2>(*minverbr.second);

}
