#include <iostream>
#include <cstring>

using std::cout;
using std::endl;

// Klassendefinition
class CString
{
    char *pString; // String
    short stringNum; // akt. String-Nummer
    inline static short count = 0; // laufende String-Nummer
public:
    // ctor
    CString();
    CString(const char *const psT);
    CString(const CString& source);
    // dtor
    ~CString();
    // Strings addieren
    void AddString(const char *const pS);
    void AddString(const CString& add);
    // String ausgeben
    void Print() const;
};

// Definition der Memberfunktionen
// 1. Konstruktor, delegation ctor!
CString::CString(): CString("")
{
}
// 2. Konstruktor (target ctor)
CString::CString(const char *const pS)
{
    // Speicherplatz für String anfordern
    // Die abschliessende binäre 0 dabei nicht vergessen!
    pString = new char[strlen(pS)+1];
    // String umkopieren
    strcpy(pString,pS);
    // Stringzähler erhöhen
    count++;
    // Und aktuelle Nummer merken
    stringNum = count;
}
// 3. Konstruktor (Kopierkonstruktor)
CString::CString(const CString& source)
{
    // Gleicher Ablauf wie beim vorherigen ctor
    pString = new char[strlen(source.pString)+1];
    strcpy(pString,source.pString);
    count++;
    stringNum = count;
}
// Destruktor
CString::~CString()
{
    // Speicher für char-Feld freigeben!
    delete [] pString;
    count--;
}
// Fügt den über den char-Zeiger übergebenen String hinzu
void CString::AddString(const char *const pS)
{
    // Zuerst Speicher für neuen String reservieren
    auto pTemp = new char[strlen(pString)+strlen(pS)+1];
    // Dann den ursprünglichen String umkopieren
    strcpy(pTemp,pString);
    // und nun den neuen String hinzufügen
    strcat(pTemp,pS);
    // Jetzt erst den ursprünglichen Speicher freigeben
    delete [] pString;
    // und Zeiger auf neuen Speicherbereich setzen
    pString = pTemp;
}
// Fügt den über die String-Referenz übergebenen String hinzu
void CString::AddString(const CString& add)
{
    // Gleicher Ablauf wie vorherige Memberfunktion
    AddString(add.pString);
}
// Ausgabe des Strings
void CString::Print() const
{
    cout << stringNum << ".String: " << pString << endl;
}

// main() Funktion
int main()
{
    // Zwei String Objekte definieren
    CString myString1("Eine Investition ");
    CString myString2;

    // Beide Strings ausgeben
    cout << "\nAusgangs-Strings\n";
    myString1.Print();
    myString2.Print();

    // Zum ersten String Objekt char-String hinzufügen
    myString1.AddString("in Wissen ");
    // Zum zweiten String Objekte char-String hinzufügen
    myString2.AddString("bringt immer noch die besten Zinsen");

    // Beide Strings ausgeben
    cout << "\nNach AddString(...)\n";
    myString1.Print();
    myString2.Print();

    // Neuen String definieren und mit dem ersten
    // CString Objekt initialisieren
    CString myString3(myString1);
    // Zeilenvorschub in String einfügen
    myString3.AddString("\n");
    // und zweites String Objekt hinzufügen
    myString3.AddString(myString2);

    // Neues String Objekt jetzt ausgeben
    cout << "\nNeuer String\n";
    myString3.Print();
}
