#include <iostream>

using std::cout;
using std::endl;

// Klassendefinition
class Rect
{
    short xPos = 0, yPos = 0; // Position
    short width = 640, height = 480; // Breite und Höhe
  public:
    Rect() = default; // ctors
    Rect(short x, short y, short w, short h);
    void PrintRect()const; // Ausgabe des Rechtecks
    Rect operator ++(int); // Verschiebt Rechteck um eine X/Y-Position
    Rect operator & (const Rect& op2) const; // Verundet zwei Rechtecke
    bool operator ==(const Rect& op2) const; // Vergleicht zwei Rechtecke
    bool operator !() const; // Prüft auf leeres Rechteck ab
};
// Definition der Memberfunktionen
// 2. Konstruktor
// Erhält als Parameter die Rechteck-Daten als 4 short Werte
Rect::Rect(short x, short y, short w, short h)
{
    xPos = x; yPos = y;
    width = w; height = h;
}
// Ausgabe der Rechteck Daten
void Rect::PrintRect() const
{
    cout << "Rechteck auf (" << xPos << "," << yPos << ") ";
    cout << "Groesse: (" << width << "," << height << ")\n";
}
// Verschiebt Rechteck um eine X/Y-Position
// Achtung Suffix-Operator!
// Darf keine const-Memberfunktion sein da das Objekt
// ja verändert wird!
Rect Rect::operator ++(int)
{
    // Hilfsobjekt zur Aufnahme der akt. Werte
    // Ruft copy-ctor auf
    Rect orig(*this);
    // Nun erst Position verändern
    xPos++;
    yPos++;
    // Ursprungswerte zurückgeben
    return orig;
}
// Verundet zwei Rechtecke
// Das daraus resultierende Rechtecke enthält die Fläche,
// die beiden Rechtecken gemeinsam ist
Rect Rect::operator & (const Rect& op2) const
{
    // Resultierendes Rechteck
    Rect result;
    // Hilfsvariablen
    short end1, end2;
    // Grösste X-Position ist resultierende X-Position
    result.xPos = (op2.xPos>xPos) ? op2.xPos : xPos;
    // X-Positionen der rechten Kante berechnen
    end1 = xPos+width;
    end2 = op2.xPos+op2.width;
    // Kleinste X-Position bestimmte Breite des result. Rechtecks
    result.width = (end1>end2) ? end2-result.xPos : end1-result.xPos;
    // Nun das gleiche Spiel mit der Y-Position und der Höhe
    result.yPos = (op2.yPos>yPos) ? op2.yPos : yPos;
    end1 = yPos+height;
    end2 = op2.yPos+op2.height;
    result.height = (end1>end2) ? end2-result.yPos : end1-result.yPos;
    // Falls keine gemeinsame Fläche vorhanden ist
    // leeres Rechteck setzen
    if ((result.width <= 0) || (result.height <= 0))
    {
        result.width = result.height = 0;
        result.xPos = result.yPos = 0;
    }
    return result;
}
// Vergleicht zwei Rechecke
bool Rect::operator ==(const Rect& op2) const
{
    // Rechtecke sind gleich wenn Koordinaten und Ausdehnung gleich sind
    if ((xPos == op2.xPos) && (width == op2.width) &&
        (yPos == op2.yPos) && (height == op2.height))
        return true;
    else
        return false;
}
// Prüft ab, ob das Rechteck eine Fläche besitzt
bool Rect::operator ! () const
{
    if ((width == 0) || (height == 0))
        return true;
    else
        return false;
}

// main () Funktion
int main()
{
    // Zwei Rect Objekte erstellen
    auto pFirstRect = new Rect(10,20,100,200);
    auto pSecondRect = new Rect(50,50,100,200);

    // Rechteckdaten ausgeben
    cout << "1. Rechteck:\n";
    pFirstRect->PrintRect();
    cout << "2. Rechteck:\n";
    pSecondRect->PrintRect();

    // Rechtecke vergleichen
    if (*pFirstRect == *pSecondRect)
        cout << "Rechtecke sind gleich\n";
    else
        cout << "Rechtecke sind unterschiedlich\n";

    // Erstes Objekt verschieben
    // Klammerung unbedingt beachten!
    (*pFirstRect)++;
    cout << "1. Rechteck um eins verschoben:\n";
    pFirstRect->PrintRect();

    // Neues Rechteck aus der gemeinsamen Fläche der beiden
    // Rechtecke bilden
    auto mergedRect = *pFirstRect & *pSecondRect;
    // Abprüfen, ob beide Rechtecke eine gemeinsame Fläche hatten
    if (!mergedRect)
        cout << "Keine Ueberschneidung der beiden Rechtecke!\n";
    else
    {
        cout << "Rechtecke ueberschneiden sich! Gemeinsame Flaeche:\n";
        mergedRect.PrintRect();
    }

    delete pFirstRect;
    delete pSecondRect;
}
