#include <iostream>
#include <cstring>

using std::cout;
using std::endl;

// Klassendefinition
class CString
{
    char *pString; // String
    short stringNum; // akt. String-Nummer
    inline static short count = 0; // laufende String-Nummer
public:
    // ctor
    CString();
    CString(const char *const psT);
    CString(const CString& source);
    // dtor
    ~CString();
    // Strings zuweisen
    CString& operator = (const char *const pS);
    CString& operator = (const CString& source);
    CString& operator = (CString&& source);
    // Strings addieren
    CString operator + (const char *const pS) const;
    CString operator + (const CString& op2) const;
    // Strings vergleichen
    bool operator == (const CString& op2) const;
    // String ausgeben
    void Print() const;
};

// Definition der Memberfunktionen
// 1. Konstruktor
CString::CString(): CString("")
{
}
// 2. Konstruktor
CString::CString(const char *const pS)
{
    // Speicherplatz für String anfordern
    // Die abschliessende binäre 0 dabei nicht vergessen!
    pString = new char[strlen(pS)+1];
    // String umkopieren
    strcpy(pString,pS);
    // Stringzähler erhöhen
    count++;
    // Und aktuelle Nummer merken
    stringNum = count;
}
// 3. Konstruktor (Kopierkonstruktor)
CString::CString(const CString& source)
{
    // Gleicher Ablauf wie beim vorherigen ctor
    pString = new char[strlen(source.pString)+1];
    strcpy(pString,source.pString);
    count++;
    stringNum = count;
}
//
// Destruktor
CString::~CString()
{
    // Speicher für char-Feld freigeben!
    delete [] pString;
    count--;
}
// Weist einem String Objekt einen char-String zu
CString& CString::operator = (const char *const pS)
{
    // Alten String löschen
    delete [] pString;
    // Neuen String anlegen
    pString = new char[strlen(pS)+1];
    strcpy(pString,pS);
    return *this;
}
// Weist einem String Objekt ein anderes String Objekt zu
CString& CString::operator = (const CString& source)
{
    // Zuweisung auf sich selbst abprüfen
    if (&source == this)
        return *this;
    else
        // Zuweisung ausfuehren
        return operator = (source.pString);
}
// Move-Operator =
// Wird aufgerufen, wenn ein temporäres CString-
// Objekt zugewiesen wird
CString& CString::operator = (CString&& source)
{
    if (this == &source)
        return *this;
    // Speicher-Ressource uebertragen
    pString = source.pString;
    // Die String-Nummer bleibt erhalten, da temp. Objekt geloescht wird
    // Quell-Objekt in gueltigen Zustand bringen
    source.pString = new char[1];
    source.pString[0] = 0;
    return *this;
}
// Fügt den über den char-Zeiger übergebenen String hinzu
CString CString::operator + (const char *const pS) const
{
    // Temp. Objekt
    CString temp;

    // Zuerst den Speicher des temp. Objekts freigeben!
    delete [] temp.pString;

    // Nun Speicher für neuen String reservieren
    temp.pString = new char[strlen(pString)+strlen(pS)+1];
    // Zuerst den ursprünglichen String umkopieren
    strcpy(temp.pString,pString);
    // dann den String des 2. Operanden hinzufügen
    strcat(temp.pString,pS);
    return temp;
}
// Fügt den über die String-Referenz übergebenen String hinzu
CString CString::operator + (const CString& op2) const
{
     return operator + (op2.pString);
}
// Vergleicht zwei String Objekte
bool CString::operator == (const CString& op2) const
{
    return (strcmp(pString,op2.pString) == 0);
}

// Ausgabe des Strings
void CString::Print() const
{
    cout << stringNum << ".String: " << pString << endl;
}

// main() Funktion
int main()
{
    // Zwei String Objekte definieren
    CString myString1("Eine Investition ");
    CString myString2("in Wissen ");

    // Beide Strings ausgeben
    cout << "\nAusgangs-Strings\n";
    myString1.Print();
    myString2.Print();

    // Neuen String definieren
    CString myString3;
    // und ihm beide Strings zuweisen
    myString3 = myString1 + myString2;
    // Neuen String ausgeben
    myString3.Print();

    // Strings nun vergleichen
    if (myString1 == myString2)
        cout << "Die Strings sind gleich\n";
    else
        cout << "Die Strings sind ungleich\n";

    // String1 gleich String2 setzen
    myString1 = myString2;
    cout << "\nStrings einander zugewiesen!\n";
    myString1.Print();
    myString2.Print();
    // und nochmals vergleichen
    if (myString1 == myString2)
        cout << "Die Strings sind gleich\n";
    else
        cout << "Die Strings sind ungleich\n";
}
