#include <iostream>
#include <cstring>
#include <cctype>

using std::cout;
using std::endl;

// Klassendefinition
class CString
{
    char *pString; // String
    short stringNum; // akt. String-Nummer
    inline static short count = 0; // laufende String-Nummer
public:
    // ctor
    CString();
    CString(const char *const psT);
    CString(const CString& source);
    // dtor
    ~CString();
    // Strings zuweisen
    CString& operator = (const char *const pS);
    CString& operator = (const CString& source);
    CString& operator = (CString&& source);
    // Strings addieren
    CString operator + (const char *const pS) const;
    CString operator + (const CString& op2) const;
    // Strings vergleichen
    bool operator == (const CString& op2) const;
    // Überladener Ausgabe- und Eingabeoperator
    friend std::ostream& operator << (std::ostream& os, const CString& op2);
    friend std::istream& operator >> (std::istream& is, CString& op2);
    // Überladener Indexoperator
    char& operator [] (const int index);
};

// Definition der Memberfunktionen
// 1. Konstruktor
CString::CString(): CString("")
{
}
// 2. Konstruktor
CString::CString(const char *const pS)
{
    // Speicherplatz für String anfordern
    // Die abschliessende binäre 0 dabei nicht vergessen!
    pString = new char[strlen(pS)+1];
    // String umkopieren
    strcpy(pString,pS);
    // Stringzähler erhöhen
    count++;
    // Und aktuelle Nummer merken
    stringNum = count;
}
// 3. Konstruktor (Kopierkonstruktor)
CString::CString(const CString& source)
{
    // Gleicher Ablauf wie beim vorherigen ctor
    pString = new char[strlen(source.pString)+1];
    strcpy(pString,source.pString);
    count++;
    stringNum = count;
}
// Destruktor
CString::~CString()
{
    // Speicher für char-Feld freigeben!
    delete [] pString;
    count--;
}
// Weist einem String Objekt einen char-String zu
CString& CString::operator = (const char *const pS)
{
    // Alten String löschen
    delete [] pString;
    // Neuen String anlegen
    pString = new char[strlen(pS)+1];
    strcpy(pString,pS);
    return *this;
}
// Weist einem String Objekt ein anderes String Objekt zu
CString& CString::operator = (const CString& source)
{
    // Zuweisung auf sich selbst abpruefen
    if (&source == this)
        return *this;
    else
        // Zuweisung ausfuehren
        return operator = (source.pString);
}
// Move-Operator =
CString& CString::operator = (CString&& source)
{
    if (this == &source)
        return *this;
    // Speicher-Ressource uebertragen
    pString = source.pString;
    // Die String-Nummer bleibt erhalten, da temp. Objekt geloescht wird
    // Quell-Objekt in gueltigen Zustand bringen
    source.pString = new char[1];
    source.pString[0] = 0;
    return *this;
}
// Fügt den über den char-Zeiger übergebenen String hinzu
CString CString::operator + (const char *const pS) const
{
    // Temp. Objekt
    CString temp;

    // Zuerst den Speicher des temp. Objekts freigeben!
    delete [] temp.pString;

    // Nun Speicher für neuen String reservieren
    temp.pString = new char[strlen(pString)+strlen(pS)+1];
    // Zuerst den ursprünglichen String umkopieren
    strcpy(temp.pString,pString);
    // dann den String des 2. Operanden hinzufügen
    strcat(temp.pString,pS);
    return temp;
}
// Fügt den über die String-Referenz übergebenen String hinzu
CString CString::operator + (const CString& op2) const
{
     return operator + (op2.pString);
}
// Vergleicht zwei String Objekte
bool CString::operator == (const CString& op2) const
{
    return (strcmp(pString,op2.pString) == 0);
}
// Der Operator[] soll den Zugriff auf ein bestimmtes Zeichen im Text
// erlauben. Ist der Index grösser als die Stringlänge wird eine
// Referenz auf die abschliessende binäre 0 zurückgegeben. Damit kann
// im Programm mit der Anweisung
// if (CStringObj[nIndex] == 0)
// abgefragt werden, ob über den String hinaus zugegriffen wird!
// Als Returnwert muss hier eine Referenz auf das entsprechende Zeichen
// angegeben werden und nicht das Zeichen selbst da sonst der Indexoperator
// nicht links vom Gleichheitszeichen stehen darf!
char& CString::operator [] (const int index)
{
    // Falls über den String hinaus zugegriffen wird
    if (index >= static_cast<int>(strlen(pString)) || (index<0))
       // Referenz auf die abschliessende binäre 0 zurückgeben
       return pString[strlen(pString)];
    // Referenz auf das gewünschte Zeichen liefern
    return pString[index];
}
// Überladene Operatorfunktion zur Ausgabe des String Objekts
std::ostream& operator << (std::ostream& os, const CString& op2)
{
    os << op2.stringNum << ".String: " << op2.pString << endl;
    // Referenz auf Stream zurückgeben
    return os;
}
// Überladene Operatorfunktion zum Einlesen eines String Objekts
std::istream& operator >> (std::istream& is, CString& op2)
{
    // Temp. Puffer für die Eingabe
    char acBuffer[80];
    // Eine Zeile einlesen
    is.getline(acBuffer,sizeof(acBuffer));
    // alten String freigeben
    delete [] op2.pString;
    // Platz für neuen String anfordern
    op2.pString = new char[strlen(acBuffer)+1];
    // String jetzt umkopieren
    strcpy(op2.pString,acBuffer);
    // Referenz auf Stream zurückgeben
    return is;
}

// main() Funktion
int main()
{
    // Zwei String Objekte definieren
    CString myString1("Eine Investition in Wissen");
    CString myString2("bringt immer noch die besten Zinsen");

    // Beide Strings ausgeben
    cout << "\nAusgangs-Strings\n" << myString1 << myString2;

    // Nun das erste String Objekt einlesen
    cout << "\nBitte neuen Text fuer erstes String Objekt eingeben:\n";
    std::cin >> myString1;

    // Kontrollausgabe
    cout << "Die Eingabe war:\n" << myString1;

    // Jetzt Eingabe in Grossbuchstaben umwandeln
    int index = 0;
    char character;
    do
    {
       character = myString1[index];
        if (character != 0)
            myString1[index++] = std::toupper(character);
    } while (character != 0);

    // Konvertierten Text ausgeben
    cout << "Alles in Grossbuchstaben:\n" << myString1;
}
