#include <queue>
#include <iostream>
#include <string>
#include <tuple>

using std::cout;
using std::endl;
using namespace std::string_literals;

// Synonym fuer Queue mit std:string
using PData = std::queue<std::string>;
// enum fuer die Frischwaren-Gruppen
enum Produkt {BUTTER, KAESE, MILCH};
// Dekodieren der Produktgruppen
const std::string prodName[] {"Butter"s, "Kaese"s, "Milch"s};

// Funktion zum Einlagern einer Ware
// lag: Feld mit den Lager fuer Frischwaren
// prod: Produkt
// anzPal: Anzahl der Paletten
// ware: Einzulagernde Ware
void Einlagern(PData lag[], enum Produkt prod, int anzPal, std::string ware)
{
    // Ware in entsprechende Lagerlinie einlagern
    for (int index=0; index<anzPal; index++)
        lag[prod].emplace(ware);
    cout << "Eingelagert " << anzPal << " Paletten \""
         << ware << "\" in Lager " << prodName[prod] << endl;
}

// Funktion zum Abholen einer Ware
// lag: Feld mit den Lager fuer Frischwaren
// prod: Frischwaren-Gruppe
// anzPal: Anzahl Paletten

void Abholen(PData lag[], enum Produkt prod, int anzPal)
{
    while ((!lag[prod].empty()) && (anzPal != 0))
    {
        cout << "Hole 1 Palette \"" << lag[prod].front()
            << "\" aus Lager " << prodName[prod] << endl;
        lag[prod].pop();
        anzPal--;
    }
}

int main()
{
    // Lager mit 3 Linien fuer die 3 verschiedenen
    // Frischwaren-Gruppen
    std::queue<std::string> lager[MILCH+1];

    // Die erste Lieferung von Waren
    // Produkt, Anzahl Palettte, Warenbezeichnung
    std::tuple<Produkt,int,std::string> lieferungen[]
    {
        std::tuple{MILCH, 3, "Weideglueck"s},
        std::tuple{KAESE, 1, "Berghof"s},
        std::tuple{MILCH, 2, "Von der Alm"s},
        std::tuple{BUTTER,2, "Landgold"s},
        std::tuple{KAESE, 2, "Zartschmelz"s},
        std::tuple{BUTTER,2, "Die Streichzarte"s},
    };

    // Gesamte Lieferung ins Lager uebernehmen
    for (auto [prod, paletten, ware]: lieferungen)
        Einlagern(lager, prod, paletten, ware);

    // Tagesbegin, Regale auffuellen
    cout << "2x Kaeseregal auffuellen:\n";
    Abholen(lager, KAESE, 2);
    cout << "4x Milchregal auffuellen\n";
    Abholen(lager, MILCH, 4);
    cout << "3x Butterregal auffuellen\n";
    Abholen(lager, BUTTER, 3);

    cout << "Neue Lieferung: 3 Paletten Butter \"Unsere Beste\"\n";
    Einlagern(lager, BUTTER, 3, "Unsere Beste"s);

    cout << "Alles muss raus:\n";
    Abholen(lager,BUTTER,INT_MAX);
    Abholen(lager,KAESE,INT_MAX);
    Abholen(lager,MILCH,INT_MAX);
}
