#include <iostream>
#include <fstream>
#include <set>
#include <string>
#include <cctype>

using std::cout;
using std::endl;
using std::string;

// Name der zu untersuchenden Textdatei
const string FILENAME("cont3.txt");

// Functor, Stringvergleich ohne Berücksichtigung der
// Gross-/Kleinschreibung
struct ignorecase
{
    bool operator()(const string& s1, const string& s2) const
    {
        auto iter1 = s1.begin();
        auto iter2 = s2.begin();
        while ((iter1!=s1.end()) && (iter2!=s2.end()))
        {
            // tolower
            char cs1 = std::tolower(*iter1);
            char cs2 = std::tolower(*iter2);
            // Buchstaben vergleichen
            if (cs1 != cs2)
                return (cs1<cs2);
            // Bei Gleichheit, nächster Buchstabe
            ++iter1;
            ++iter2;
        }
        return false;
    }
};

// main() Funktion
int main()
{
    // Zu untersuchende Datei öffnen
    std::ifstream infile(FILENAME);
    if (!infile)
    {
        cout << "Fehler beim Oeffnen der Datei " << FILENAME << endl;
        exit(1);
    }

    // Set für Wörter aus Datei anlegen
    // Ignoriert Gross/Kleinschreibung beim Einfügen über Functor ignorecase()
    std::set<string,ignorecase> dictionary;

    // Komplette Datei einlesen
    cout << "Woerter ins Woerterbuch eintragen:\n";
    do
    {
        // Wort aus Datei einlesen
        string word;
        infile >> word;
        if (infile.eof())
            continue;
        // Punkt und Komma am Wortende ignorieren
        auto interpunktion = *(word.end()-1);
        if ((interpunktion == '.') ||
            (interpunktion == ','))
        {
            cout << word << " ---> ";
            word.erase(word.end()-1);
            cout << word << endl;
        }
        // und im Set ablegen. Falls Wort schon vorhanden, Meldung ausgeben
        if (auto [pos,success] = dictionary.insert(word); !success)
            cout << "Wort '" << *pos << "' schon vorhanden!\n";
    }while (!infile.eof());

    // Alle Wörter aus Datei ausgeben
    cout << "\nIn der Datei enthaltene Worte:\n";
    for (const auto& word: dictionary)
        cout << word << ',';
    cout << endl;

    // Nun alle Wörter mit einem bestimmten Buchstaben ausgeben
    // Gesuchter String
    string toSearch("e");
    // Iterator auf ersten übereinstimmenden Eintrag
    auto lowerPos = dictionary.lower_bound(toSearch);
    // Iterator auf ersten nicht mehr übereinstimmenden Folgeeintrag
    auto upperPos = dictionary.upper_bound(toSearch);
    // Alle gefunden Wörter ausgeben
    cout << "\nAlle Woerter mit dem Buchstaben 'e':\n";
    for (auto iter=lowerPos; iter!=upperPos; ++iter)
        cout << *iter << ' ';
    cout << "\n\nAnzahl der Woerter (ohne Duplikate): " << dictionary.size() << endl;
}
