#include <iostream>
#include <string>
#include <map>
#include <iterator>

using std::cout;
using std::endl;
using std::string;

// Adressbuch Klasse
class AddressBook
{
    // Adressdaten, ohne Name
    // ACHTUNG! Falls Adressdaten auch Zeiger oder weiter Objekte
    // enthalten, so muss der copy-ctor, der = Operator und
    // der dtor noch definiert werden. Die string-Objekte sind
    // direkt kopierbar (per Zuweisung)
    using entry = std::pair<string, string>;
    // Map für Adressbuch-Einträge, Schlüssel ist Name
    // Die Nutzdaten sind in der Struktur Entry zusammengefasst
    std::map<string,entry> theBook;
  public:
    // ctor
    AddressBook() = default;
    // dtor
    ~AddressBook() = default;
    // Neue Adresse aufnehmen
    void Insert(const string& name, const string& phone, const string& email);
    // Gibt einzelne Adresse aus
    void PrintAddress(const string& name);
    // Ändert Adressbuch-Eintrag
    bool Update(const string& name, const string& phone, const string& email);
    // Adresse entfernen
    void Remove(const string& name);
    // Liefert Anzahl der Adressbuch-Einträge
    size_t size()
    {
        return theBook.size();
    }
    // Gibt gesamtes Adressbuch aus
    friend std::ostream& operator << (std::ostream& os, AddressBook& abook);
};
// Ausgabe einer Adresse
void AddressBook::PrintAddress(const string& name)
{
    // Suche nach Namen

    // Falls Name nicht im Adressbuch
    if (auto iter = theBook.find(name); iter == theBook.end())
        cout << "Kein entsprechender Eintrag gefunden!\n";
    else
    {
        // Sonst Adressdaten ausgeben, Iterator zeigt auf gefunden Eintrag
        cout << "Name: " << iter->first << '\n'
             << "Tel : " << iter->second.first
             << ", Email: " << iter->second.second << endl;
    }
}
// Neuen Eintrag ins Adressbuch aufnehmen
void AddressBook::Insert(const string& name, const string& phone, const string& email)
{
    // Map benötigt zum Einfügen ein entsprechendes pair
    std::pair<string,entry> toInsert(name,entry(phone,email));
    // pair mit den Daten einfügen
    auto success = theBook.insert(toInsert);
    // Da wir nur eine Map haben sind keine doppelten Einträge erlaubt
    if (!success.second)
        cout << name << " konnte nicht eingefuegt werden!\n";
}
// Eintrag aus Adressbuch entfernen
void AddressBook::Remove(const string& name)
{
    // Eintrag suchen
    if (auto iter = theBook.find(name); iter == theBook.end())
        cout << name << " nicht im Adressbuch vorhanden!\n";
    else
        // Wenn gefunden, dann löschen
        theBook.erase(iter);
}
// Eintrag aendern
bool AddressBook::Update(const string& name, const string& phone_, const string& email_)
{
    bool success = false;
    // Eintrag suchen und falls gefunden
    if (auto iter = theBook.find(name); iter != theBook.end())
    {
        // Telefon-Nummer und Email korrigieren
        iter->second.first = phone_;
        iter->second.second = email_;
        success = true;
    }
    return success;
}
// Ausgabe des gesamten Adressbuchs
// I.A. kann noch kein const-Objekt übergeben werden da hierfür ein anderer,
// (noch nicht behandelter) Iteratortyp (const_iterator) benötigt wird
std::ostream& operator << (std::ostream& os, AddressBook& abook)
{
    // Komplettes Adressbuch durchlaufen und ausgeben
    for (auto [name, data]: abook.theBook)
    {
        os << "Name: " << name << '\n'
           << "Tel : " << data.first
           << ", Email: " << data.second << endl;
    }
    return os;
}

// main() Funktion
int main()
{
    // Adressbuch dynamisch anlegen
    AddressBook *pAddr = new AddressBook;

    // Einige Einträge hinzufügen
    pAddr->Insert("Emil Maier", "0111/4545", "ema@sonst.net");
    pAddr->Insert("Agathe Maier", "0111/4545", "agma@sonst.net");
    pAddr->Insert("Franzi Xelsbrot", "08888/1234", "frax@kostnix.de");
    pAddr->Insert("Gustav Gans", "09999/1111", "guga@lucky.com");

    // Komplettes Adressbuch ausgeben
    cout << "Adressbuch enthaelt " << pAddr->size() << " Eintraege\n";
    cout << *pAddr;

    // Eintrag abändern
    cout << "Aendere Telefon-Nummer von Gustav Gans:\n";
    pAddr->Update("Gustav Gans", "09999/7777", "guga@lucky.com");
    // Und nur geänderten Eintrag ausgeben
    cout << "Neuer Eintrag ist:\n";
    pAddr->PrintAddress("Gustav Gans");

    // Eintrag entfernen
    cout << "Entferne Franzi Xelsbrot:\n";
    pAddr->Remove("Franzi Xelsbrot");

    // Nochmals komplettes Adressbuch ausgeben
    cout << "Neues Adressbuch enthaelt " << pAddr->size() << " Eintraege:\n";
    cout << *pAddr;

    // Adressbuch löschen
    delete pAddr;
}
