#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <tuple>
#include <algorithm>

using std::cout;
using std::endl;

int main()
{
    // Tuple fuer Mitgliedsdaten: Name, Eintragsdatum (Tag, Monat, Jahr)
    using data = std::tuple<std::string, unsigned short, unsigned short, unsigned short>;
    // Vektor fuer Mitgliedsdaten
    std::vector<data> mitglieder;

    // Datei mit Mitgliederliste oeffnen
    std::ifstream infile("mitglieder.txt");
    if (!infile)
    {
        cout << "Mitgliederliste nicht gefunden!";
        exit(1);
    }

    // Daten aus Datei einlesen
    do
    {
        unsigned short tag, monat;
        unsigned short jahr;
        std::string name;

        // Name einlesen
        getline(infile,name,',');
        if (!infile.eof())
        {
            // Komma ueberspringen
            infile.ignore(1);
            // Datum einlesen in getrennten Variablen
            // Punkt jeweils ueberspringen
            infile >> tag;
            infile.ignore(1);
            infile >> monat;
            infile.ignore(1);
            infile >> jahr;
            infile.ignore();
            // Daten in Vektor ablegen
            mitglieder.emplace_back(name,tag,monat,jahr);
        }

    } while (!infile.eof());
    // Datei schliessen
    infile.close();

    // Mitgliederliste ausgeben
    cout << "Mitgliederliste unsortiert:\n";
    for (auto [n,t,m,j]: mitglieder)
        cout << n << ", " << t << '.' << m << '.' << j << endl;

    // Mitgliederliste nach Eintrittdatum sortieren
    // Vereinfachte Berechnung des Eintrittstag
    std::sort(mitglieder.begin(), mitglieder.end(),
              [] (data mitgl1, data mitgl2)
              {
                  auto [name1,tag1,monat1,jahr1] = mitgl1;
                  auto [name2,tag2,monat2,jahr2] = mitgl2;
                  return (jahr1*365 + 31*monat1 + tag1 <
                          jahr2*365 + 31*monat2 + tag2);
              }
              );
    cout << "\nMitgliederliste sortiert:\n";
    for (auto [name, tag, monat, jahr]: mitglieder)
        cout << name << ", " << tag <<'.' << monat << '.' << jahr << endl;
}
