#include <iostream>
#include <limits>
#include <typeinfo>

using std::cout;
using std::endl;

// Basisklassen-Template für 'normalen' Drehschalter
// Default-Datentyp des Button ist int
template <typename T=unsigned short>
class SpinButton
{
  protected:
    T value; // akt. Wert
    T minValue; // min. Wert
    T maxValue; // max. Wert
  public:
    SpinButton()
    {
        minValue = std::numeric_limits<T>::min();
        maxValue = std::numeric_limits<T>::max();
        value = minValue;
    }
    void ShowProperties()
    {
        cout << "Datentyp: " << typeid(T).name() << '\n';
        cout << "Min/Max: (" << minValue << '/' << maxValue;
        cout << "), akt. Wert: " << value << endl;
    }
    void StepUp()
    {
        if (value < maxValue)
           value++;
    }
    void StepDown()
    {
        if (value > minValue)
            value--;
    }
};

// Von der Template-Klasse SpinButton abgeleitete Klasse
// ImageSpinButton. Datentyp des Werts von SpinButton ist
// unsigned int. Die in der Basisklasse definierte Eigenschaft
// value dient als Imagelisten-Index.
class ImageSpinButton: public SpinButton<>
{
    const char* const *imList; // ACHTUNG! char-Zeiger Feld!
  public:
    ImageSpinButton(const char* const il[]): imList(il)
    {
        // Anzahl der Images in der Liste bestimmen
        maxValue = minValue;
        while(*(il+maxValue) != nullptr)
            maxValue++;
        maxValue--;
    }
    void ShowProperties()
    {
        cout << "Datentyp: " << typeid(*this).name() << '\n';
        cout << "Min/Max: (" << minValue << '/' << maxValue;
        cout << "), akt. Image: " << imList[value] << endl;
    }
};

// main() Funktion
int main()
{
    // Spinbutton mit Default-Datentyp definieren
    SpinButton<short> intSpin;
    intSpin.ShowProperties();
    intSpin.StepUp();
    intSpin.ShowProperties();

    // Imageliste für Image-Spinbutton
    const char* const imageList1[]
          {"Image one","Image two","Image three",nullptr};
    // Image-Spinbutton definieren
    ImageSpinButton ImageSpin(imageList1);
    ImageSpin.ShowProperties();
    ImageSpin.StepUp();
    ImageSpin.ShowProperties();
    ImageSpin.StepDown();
    ImageSpin.ShowProperties();
    ImageSpin.StepDown();
    ImageSpin.ShowProperties();

}
