#include <fstream>
#include <iostream>
#include <sstream>
#include <limits>
#include <cstdlib>
#include <typeinfo>
#include <string>

using std::cout;
using std::endl;
using std::string;

// SaveFile implementiert eine Klasse zum sicheren Schreiben
// und Lesen von Daten aus eine Datei. Beim Einlesen der Daten
// wird der in der Datei mit abgelegte Datentyp mit dem einzulesenden
// Datentyp verglichen. Sind die Datentypen unterschiedlich,
// so wird eine Exception ausgelöst
class SaveFile
{
    std::fstream ioFile;
public:
    // ctor, öffnet Datei zum Lesen und Schreiben
    SaveFile(const std::string& fileName);
    // dtor, schliesst Datei
    ~SaveFile();
    // Reset setzt den Schreib- und Lesezeiger zurück
    void Reset();
    // Member-Template zum Schreiben des Datum
    template <typename T> void Write(const T& value);
    // Member-Template zum Einlesen eines Datums
    template <typename T> void Read(T& value);
};
// ctor, öffnet Datei zum Lesen und Schreiben
SaveFile::SaveFile(const std::string& fileName)
{
    ioFile.open(fileName,std::ios::in|std::ios::out|std::ios::trunc);
    if (!ioFile)
       throw string("Fehler beim Datei oeffnen!");
}
// dtor, schliesst Datei
SaveFile::~SaveFile()
{
    ioFile.close();
}
// Reset, setzt den Schreib- und Lesezeiger zurück
void SaveFile::Reset()
{
    ioFile.seekg(0);
    ioFile.seekp(0);
}
// Member-Template zum Schreiben des Datum
// Vor dem eigentlichen Datum wird der Datentyp mit
// in die Datei geschrieben. Trennzeichen zwischen
// Datentyp und Datum ist '@'
template <typename T>
void SaveFile::Write(const T& value)
{
    ioFile << typeid(T).name() << '@' << value << std::endl;
}
// Member-Template zum Einlesen eines Datums
template <typename T>
void SaveFile::Read(T& value)
{
    // Datentyp und Datum einlesen
    string inputLine;
    std::getline(ioFile, inputLine);
    // Suche nach @ um Datentyp zu extrahieren
    auto delimiterPos = inputLine.find('@');
    if (delimiterPos == string::npos)
       throw string("Datentyp konnte nicht bestimmt werden!");
    // Datentyp extrahieren
    string inputDType;
    inputDType.assign(inputLine,0,delimiterPos);
    // Falls einzulesender Datentyp und Dateidatentyp nicht
    // identisch sind, Exception auslösen
    if (inputDType != typeid(T).name())
    {
        string err("Falscher Datentyp -> ist: ");
        err += inputDType + ", soll: " + typeid(T).name();
        throw err;
    }
    // Datentyp ueberspringen
    // ASCII-Datum nach binär konvertieren
    std::istringstream is(&inputLine[delimiterPos+1]);
    is >> value;
    if (is.fail())
       throw string("Fehler bei der Datenkonvertierung!");
}

// main() Funktion
int main()
{
    // Datei-Objekt definieren
    SaveFile testFile("test.dat");
    // 3 Daten in Datei schreiben
    char cVal = 'A';
    long double ldVal = 1.0;
    std::string anyString("Dies ist ein String");
    testFile.Write(cVal);
    testFile.Write(ldVal);
    testFile.Write(anyString);
    // Geschriebene Daten zur Kontrolle ausgeben
    cout << "Geschriebene Daten: " << cVal
         << ',' << ldVal << ',' << anyString << endl;
    // Zur Kontrolle Daten auf einen anderen Wert setzen
    cVal = 0;
    ldVal = 0;
    anyString = "";
    // Dateizeiger zurücksetzen
    testFile.Reset();
    // Nun versuchen alle Daten wieder einzulesen
    // Zum Testen mal die 1. oder 2. Zeile auskommentieren
    try
    {
        testFile.Read(cVal);
        testFile.Read(ldVal);
        testFile.Read(anyString);
    }
    catch (const string& errMsg)
    {
        cout << "Fehler: " << errMsg << endl;
        exit(1);
    }
    // Eingelesen Daten zur Kontrolle ausgeben
    cout << "Eingelesene Daten: " << cVal
         << ',' << ldVal << ',' << anyString << endl;
}
