#include <iostream>
#include <fstream>
#include <string>

using std::cout;
using std::endl;

// Allgemeines Template zur Serialisierung
template <typename T1, typename T2>
class Serialize
{
    T1& outStream;  // Ausgabestream
    T2 value;       // Wert
  public:
    // ctor
    Serialize(T1& out, const T2& v):
        outStream(out), value(v)
    {}
    // Daten ausgeben
    void Transfer()
    {
        T2 toTransfer = value;          // Wert zwischenspeichern
        outStream << std::hex;          // auf hex umstellen
        // Alle Bytes ausgeben
        for (size_t index=0; index<sizeof(T2); index++)
        {
            outStream << (toTransfer & 0xff) << ',';
            toTransfer >>= 8;
        }
        outStream << endl;
    }
};

// Spezielles Template zur Serialisierung eines strings
template <typename T1>
class Serialize<T1, std::string>
{
    T1& outStream;      // Ausgabestream
    std::string value;  // Wert
  public:
    // ctor
    Serialize(T1& out, const std::string& v):
        outStream(out), value(v)
    { }
    // Daten ausgeben
    void Transfer()
    {
        // string zeichenweise durchlaufen und ausgeben
        for (size_t index=0; index<value.length(); index++)
        {
            outStream << value[index] << ',';
        }
        outStream << endl;
    }
};

// main() Funktion
int main()
{
    // Variablen definieren/initialisieren
    long lVar = 0x11223344L;
    std::string sVar {"Ein Text"};

    // long Wert auf Standardausgabe
    Serialize<std::ostream, decltype(lVar)> numVar(cout, lVar);
    numVar.Transfer();
    // string auf Standardausgabe
    Serialize<std::ostream, std::string> stringVar(cout, sVar);
    stringVar.Transfer();

    // Ausgabedatei oeffnen
    std::ofstream outFile("tspec.dat");
    if (!outFile)
    {
        cout << "Datei konnte nicht geoeffnet werden!\n";
    }
    else
    {
        Serialize<std::ofstream,long> fnumVar(outFile, lVar);
        fnumVar.Transfer();
        Serialize<std::ofstream,std::string> fstringVar(outFile, sVar);
        fstringVar.Transfer();
        outFile.close();
    }
}
