#include <iostream>
#include <iomanip>
#include <string>
#include <algorithm>

using std::cout;
using std::endl;

// Klassentemplate für allgemeine Matrix
// T = Datentyp der Elemente
// r = Anzahl der Zeilen
// c = Anzahl der Spalten
template <typename T, int r, int c>
class matrix
{
    T **data; // Zeiger-Zeiger(!) auf dyn. 2D-Feld
    int col; // Spalten
    int row; // Zeilen
  public:
    // ctor, reserviert 2D-Feld dynamisch
    matrix()
    {
        row = r; // Zeilen/Spalten merken
        col = c;
        // 2D-Feld dyn. reservieren
        data = new T*[row];
        for (auto i=0; i<row; i++)
            data[i] = new T[col];
    }
    // Memberfunktionen zum setzen eines Matrix-Elements
    // Zeilen/Spalten-Nr. beginnen ab 1!
    void setData(int row_, int col_, T val)
    {
        if ((col_<1) || (col_>col) ||
            (row_<1) || (row_>row))
            throw "Falsch Zeile oder Spalte";
        data[row_-1][col_-1] = val;
    }
    // Matrix transponieren (Zeilen/Spalten vertauschen)
    void Transpose()
    {
        cout << "Transponiere rechteckige Matrix\n";
        // Neues 2D-Feld anlegen mit vertauschter Grösse
        // Spalte->Zeile, Zeile->Spalte
        T **temp;
        temp = new T*[col];
        for (auto i=0; i<col; i++)
            temp[i] = new T[row];
        // Matrix transponieren
        for (auto ri=0; ri<row; ri++)
            for (int ci=0; ci<col; ci++)
                temp[ci][ri] = data[ri][ci];
        // Alte Matrix nun löschen
        for (auto i1=0; i1<row; i1++)
            delete [] data[i1];
        delete [] data;
        // Zeiger auf neue Matrix abspeichern
        data = temp;
        // Zeilen/Spalten Anzahl vertauschen
        std::swap(col,row);
    }
    // Matrix ausgeben
    void Print() const
    {
        for (auto ri=0; ri<row; ri++)
        {
            for (auto ci=0; ci<col; ci++)
                cout << std::setw(5) << data[ri][ci];
            cout << endl;
        }
    }
};

// Partiell spezialisierte Matrix für eine quadratische
// Matrix (Zeilen=Spalten)
template <typename T, int s>
class matrix<T,s,s>
{
    T **data; // dyn. 2D-Feld
    int size; // Grösse der quadratischen Matrix
  public:
    // ctor, reserviert dyn. 2D-Feld
    matrix()
    {
        size = s;
        data = new T*[size];
        for (auto i=0; i<size; i++)
            data[i] = new T[size];
    }
    // Memberfunktionen zum setzen eines Matrix-Elements
    // Zeilen/Spalten-Nr. beginnen ab 1!
    void setData(int row_, int col_, T val)
    {
        if ((col_<1) || (col_>size) ||
            (row_<1) || (row_>size))
            throw "Falsch Zeile oder Spalte";
        data[row_-1][col_-1] = val;
    }
    // Matrix transponieren
    // Bei quadratischen Matrizen muss kein temporäres 2D-Feld
    // angelegt werden; es können direkt die Matrix-Elemente
    // vertauscht werden
    void Transpose()
    {
        cout << "Transponiere quadratische Matrix\n";
        for (auto ri=0; ri<size-2; ri++)
            for (auto ci=ri+1;ci<=size-1; ci++)
                std::swap(data[ci][ri],data[ri][ci]);
    }
    // Matrix ausgeben
    void Print() const
    {
        for (auto ri=0; ri<size; ri++)
        {
            for (auto ci=0; ci<size; ci++)
                cout << std::setw(5) << data[ri][ci];
            cout << endl;
        }
    }
};

// main() Funktion
int main()
{
    // Rechteckige Matrix anlegen
    matrix<short,3,4> rectMat;
    // Und mit Werten belegen
    for (auto r=1; r<=3; r++)
        for (auto c=1; c<=4; c++)
            rectMat.setData(r,c,c*10+r);
    // Matrix ausgeben
    cout << "Ausgangsmatrix:\n";
    rectMat.Print();
    // Matrix transponieren und ausgeben
    rectMat.Transpose();
    cout << "Transponierte Matrix:\n";
    rectMat.Print();

    // Quadratische Matrix anlegen
    matrix<double,4,4> sqrMat;
    // Und mit Werten belegen
    for (auto r=1; r<=4; r++)
        for (auto c=1; c<=4; c++)
            sqrMat.setData(r,c,double(r*100+c)/10.0);
    // Matrix ausgeben
    cout << "\nAusgangsmatrix:\n";
    sqrMat.Print();
    // Matrix transponieren und ausgeben
    sqrMat.Transpose();
    cout << "Transponierte Matrix:\n";
    sqrMat.Print();
}
