#include <iostream>
#include <string>

using std::cout;
using std::endl;
using std::string;

// Definition der Klasse CGraphic
// CGraphic ist eine abstrakte Klasse da sie die 'pure virtual'
// Memberfunktion Draw() enthält.
// CGraphic speichert die allen Grafikelementen gemeinsamen
// X/Y Koordinaten.
class CGraphic
{
  protected:
    short nXPos, nYPos; // Koordinaten der Grafik
    CGraphic(short nX, short nY); // protected-Konstruktor
  public:
    virtual ~CGraphic() = default; // virtueller dtor!
    virtual void Draw() const = 0;
};
// Definition der Memberfunktionen
// Konstruktor
CGraphic::CGraphic(short nX, short nY)
{
    // Koordinaten der Grafik ablegen
    nXPos = nX;
    nYPos = nY;
    cout << "Konstruktor CGraphic\n";
}

// Definition der Klasse CCircle
// CCircle ist abgeleitet von CGraphic und enthält als
// zusätzliches Datum den Radius eines Kreises
class CCircle: virtual public CGraphic
{
    short nRadius; // Kreisradius
  public:
    CCircle (short, short, short);
    void Draw() const;
};
// Definition der Memberfunktionen
// Konstruktor
CCircle::CCircle(short nX, short nY, short nR):
    CGraphic(nX, nY) // Konstruktor Basisklasse
{
    // Radis ablegen
    cout << "Konstruktor CCircle\n";
    nRadius = nR;
}
// Gibt die Kreisdaten aus
void CCircle::Draw() const
{
    cout << "Kreis-Parameter\n";
    cout << "Position: " << nXPos << "," << nYPos << endl;
    cout << "Radius : " << nRadius << endl;
}

// Definition der Klasse CBar
// CBar ist abgeleitet von CGraphic und enthält als
// zusätzliches Datum die Breite und Höhe des Rechtecks
class CBar: virtual public CGraphic
{
    short nWidth, nHeight; // Grösse des Rechtecks
  public:
    CBar (short, short, short, short);
    void Draw() const;
};
// Definition der Memberfunktionen
// Konstruktor der Klasse CBar
CBar::CBar(short nX, short nY, short nW, short nH):
    CGraphic(nX, nY) // Konstruktor Basisklasse
{
    // Ausdehnung ablegen
    cout << "Konstruktor CBar\n";
    nWidth = nW;
    nHeight = nH;
}
// Gibt die Daten des Rechtecks aus
void CBar::Draw() const
{
    cout << "Rechteck-Parameter\n";
    cout << "Position: " << nXPos << "," << nYPos << endl;
    cout << "Groesse : " << nWidth << "," << nHeight << endl;
}

// Definition der Klasse CCBar
// CCBar ist abgeleitet von CCircle und CBar und damit auch
// von CGraphic
class CCBar: public CCircle, public CBar
{
    string sText;
  public:
    CCBar(short, short, short, const char *const);
    void Draw() const;
};
// Definition der Memberfunktionen
// Konstruktor von CCBar
// Beachten Sie die Aufrufe der Konstruktoren der Basisklassen.
// Obwohl CCBar nicht direkt von CGraphic abgeleitet ist muß der
// Konstruktor von CGraphic gesondert aufgerufen werden
CCBar::CCBar(short nX, short nY, short nW, const char *const pszT):
    CGraphic(nX, nY), // ctor virtuelle Basisklasse
    CCircle(nX, nY, nW>>1), // ctor Basisklasse
    CBar(nX, nY, nW, nW), // ctor Basisklasse
    sText(pszT) // string initialisieren
{
    cout << "Konstruktor CCBar\n";
}
// Gibt Daten des beschrifteten Rechtecks aus
// Ruft die Memberfunktionen Draw() der Klassen CBar und CCircle auf
// Beachten Sie, daß unbedingt die Angabe des Zugriffsspezifizieres
// beim Aufruf der Memberfunktionen notwendig ist
void CCBar::Draw() const
{
    cout << "CCBar besteht aus:\n";
    CCircle::Draw();
    CBar::Draw();
    cout << "Beschriftung: " << sText << endl;
}

// main() Funktion
int main()
{
    // Feld für 3 Zeiger auf Grafikobjekte definieren
    // Das Feld nimmt Basisklassenzeiger auf!
    constexpr auto SIZE = 3;
    CGraphic *pCObject[SIZE];
    // Kreis erstellen und Zeiger ablegen
    cout << "--- Definition CBar ---\n";
    pCObject[0] = new CBar(10,10,20,20);
    // Rechteck erstellen und Zeiger ablegen
    cout << "--- Definition CCircle ---\n";
    pCObject[1] = new CCircle(40,40,20);
    // Beschriftetes Rechteck erstellen und Zeiger ablegen
    cout << "--- Definition CCBar ---\n";
    pCObject[2] = new CCBar(50,50,100,"Rechteck mit Text");

    // Alle Grafiken ausgeben und dann wieder löschen
    for (auto iIndex=0; iIndex<SIZE; iIndex++)
    {
        pCObject[iIndex]->Draw();
        delete pCObject[iIndex];
    }
}
