#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <tuple>
#include <string>

using std::cout;
using std::endl;

// Datei mit Daten
constexpr const char* FNAME = "aktien.csv";

int main()
{
    // tuple fuer Aktiendaten
    // Unternehmen, aktuelle Kurs, hoechster Kurs, Datum Hoechststand
    using data = std::tuple<std::string, float,float,std::string>;
    // Vektor fuer Aktiendaten
    std::vector<data> vect;

    // Datei oeffnen und Fehler abfangen
    std::ifstream infile(FNAME);
    if (!infile)
    {
        cout << "Datei nicht gefunden: " << FNAME << endl;
        exit(1);
    }
    // Daten einlesen
    do
    {
        // Eingelesen Zeile
        std::string line;
        // Konvertierungsstream
        std::istringstream is;
        // Aktiendaten
        std::string unternehmen, datum;
        float aktKurs, hochKurs;
        // Zeile einlesen
        // Bei erreichen des Dateiendes den Rest überspringen
        if(getline(infile,line);infile.eof())
            continue;
        // Zeile in Konvertierungsstream uebernehmen
        is.str(line);
        // Unternehmen extrahieren
        getline(is,unternehmen,';');
        // aktueller Kurs
        is >> aktKurs;
        // Semikolon ueberspringen
        is.ignore(1);
        // Hoechster Kurs
        is >> hochKurs;
        // Semikolon ueberspringen
        is.ignore(1);
        // Datum
        getline(is,datum);
        // Daten in Vektor uebernehmen
        vect.push_back({unternehmen, aktKurs, hochKurs, datum});

    } while (!infile.eof());
    // Datei wieder schliessen
    infile.close();

    int mindex = 0;         // Index des groessen Verlusts
    float mdiff = 1.0f;     // maximaler Verlust
    int index = 0;          // Indexzaehler
    // Vektor durchlaufen und Daten ablegen (structured binding!)
    for (const auto& [untern,akt,hoch,dat]: vect)
    {
        // Daten ausgeben
        cout << untern << " akt: " << akt << ", max: "
            << hoch << " am " << dat << endl;
        // Verlust in Prozent mit max. Verlust vergleichen
        if (akt/hoch < mdiff)
        {
            // Neuer max. Verlust gefunden und merken
            mdiff = akt/hoch;
            mindex = index;
        }
        // Index erhoehen
        index++;
    }
    // Daten des max. Verlust aus Vektor holen
    auto [untern,akt,hoch,dat] = vect[mindex];
    // Und ausgeben
    cout << "\nGroesster Verlust (%): " << untern << ' '
        << static_cast<int>((1.0f-mdiff)*100) << " Prozent" << endl;
}
